#include "Philosopher.hh"
#include "Guard.hh"

#include <ctime>
#include <cmath>
#include <iostream>
using namespace std;

Philosopher::Philosopher(int id, Chopstick* left, Chopstick* right)
: Thread(true),
  id_(id),
  left_(left),
  right_(right),
  stop_called_(false)
{}

bool Philosopher::is_stopped() {
	Guard guard(stop_mutex_);
	return stop_called_;
}

void Philosopher::stop() {
	Guard guard(stop_mutex_);
	stop_called_=true;
}

int Philosopher::random_sleep() {
	return static_cast<int>(SLEEP_MAX*(static_cast<double>(rand())/RAND_MAX));
}

void Philosopher::thinking() {
	int t=random_sleep();
	cerr << "Philosopher(" << id_ << ") going to think for (" << t << ") seconds..." << endl;
	sleep(t);
	cerr << "Philosopher(" << id_ << ") thinking done..." << endl;
}

void Philosopher::eating() {
	int t=random_sleep();
	cerr << "Philosopher(" << id_ << ") going to eat for (" << t << ") seconds..." << endl;
	sleep(t);
	cerr << "Philosopher(" << id_ << ") eating done..." << endl;
}

void Philosopher::run() {
	cerr << "Philosopher(" << id_ << ") started..." << endl;
	while(!is_stopped()) {
		thinking();
		right_->get();
		left_->get();
		eating();
		right_->put();
		left_->put();				
	}
	cerr << "Philosopher(" << id_ << ") stopped..." << endl;
}



