#include "Thread.hh"
#include "Guard.hh"
#include "ThreadError.hh"

Thread::Thread(bool joinable) 
	: thread_id_(0),
	  is_started_(false),
	  joinable_(joinable)
{}

Thread::~Thread() {
	if(joinable_) {
		pthread_detach(thread_id_);
	}
}

void* Thread::internal_run(void* arg) {
	Thread* thread=static_cast<Thread*>(arg);
	thread->run();
	return NULL;
}

void Thread::start() {
	Guard guard(mutex_);

	if(is_started_){
		throw ThreadError("thread already started...");
	}
	
	pthread_attr_t attr;
	int res=pthread_attr_init(&attr);
	if(res!=0) {
		throw ThreadError("can't init thread attr...");
	}
	
	res=pthread_attr_setdetachstate(&attr,
		(joinable_?PTHREAD_CREATE_JOINABLE:PTHREAD_CREATE_DETACHED));
	if(res!=0) {
		throw ThreadError("can't set detache state...");
	}

	res=pthread_create(&thread_id_,&attr,
		Thread::internal_run,this);
	if(res!=0) {
		throw ThreadError("can't create thread...");
	}
	is_started_=true;
}

bool Thread::is_started() {
	Guard guard(mutex_);
	return is_started_;
}

bool Thread::is_joinable() {
	Guard guard(mutex_);
	return joinable_;
}

void Thread::join() {
	if(!is_started()) {
		throw ThreadError("can't join not started thread...");
	}
	if(!is_joinable()) {
		throw ThreadError("can't join detached thread...");
	}

	pthread_join(thread_id_, NULL);

	Guard guard(mutex_);
	joinable_=false;
}



